<?php

namespace App\Http\Controllers;

use App\Http\Requests\OrderValidation;
use Illuminate\Http\Request;
use App\Model\Shopify\Product;
use OhMyBrew\BasicShopifyAPI;
use Config;
use stdClass;

class OrderController extends Controller
{
    private $productObj;

     /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct() {
        $this->productObj = new Product();
    }

    // To create order view
    public function createOrder($status = []){
        return view('ajax_inflenceOrder');
    }

    // To store the order in shopify
    public function saveOrder() {
        $message = "Endpoint parameters are missing";
        $status = "Failed";
        
        if (empty($_REQUEST['data'])){
            $response = array(
                'status' => $status,
                'errorMessage' => $message,
            );
            echo json_encode($response);
            die();
        } else {
            $data = json_decode($_REQUEST['data'],true);
            $lineItemStatus = 0;
            $shippingStatus = 0;
            if (empty($data['order']['email']) || empty($data['order']['product_id']) || empty($data['order']['variant_id'])){
                $response = array(
                    'status' => $status,
                    'errorMessage' => $message,
                );
                echo json_encode($response);
                die();
            } else {
                
                if (isset($data['order']['line_items'])){
                    foreach ($data['order']['line_items'] as $lineItems){
                        if ((empty($lineItems['variant_id']) || empty($lineItems['product_id']) || empty($lineItems['name']) || empty($lineItems['value'])) && empty($lineItems['utm'])){
                            $lineItemStatus = 1;
                        }
                    }
                } else {
                    $lineItemStatus = 1;
                }

                if (empty($data['order']['line_items']) || $lineItemStatus == 1){
                    $response = array(
                        'status' => 'Failed',
                        'errorMessage' => 'Missing required parameters in the line item properties',
                    );
                    echo json_encode($response);
                    die();
                } 

                if (!empty($data['order']['shipping_address'])){
                    $shippingaddress = $data['order']['shipping_address'];
                    if(empty($shippingaddress['first_name']) || empty($shippingaddress['last_name']) || empty($shippingaddress['address1']) || empty($shippingaddress['city']) || empty($shippingaddress['country']) || empty($shippingaddress['state'])){
                        $shippingStatus = 1;
                    }
                } else {
                    $shippingStatus = 1;
                }

                if ($shippingStatus == 1){
                    $response = array(
                        'status' => 'Failed',
                        'errorMessage' => 'Missing required parameters in the Shipping Address',
                    );
                    echo json_encode($response);
                    die();
                }

            }

        }

        $requestData = json_decode($_REQUEST['data'],true);
        $api = new BasicShopifyAPI();
        $password   = Config::get('shopify.password');
        $shopDomain = Config::get('shopify.shopDomain');
        $api->setShop($shopDomain);
        $api->setAccessToken($password);
        $setOrderRequest = $this->setOrderRequest($requestData);
        $params['order'] = $setOrderRequest;
        $result = $api->rest('POST', htmlentities('/admin/orders.json'), $params); 
      
        if ($result->errors){
            $response = array(
                'status' => $status,
                'errorMessage' => $result->errors,
            );
            echo json_encode($response);
            die();
        } else {
            $response = array(
                'status' => 'success',
                'successMessage' => $result->body,
            );
            echo json_encode($response);
            die();
        }
        
    }

    // To set the request for order and draft order
    public function setOrderRequest($reqObj){
        
        $request = [];
        $lineProperties = [];
        $prod  = $this->setProductDetails();
        
        if (isset($reqObj['order']['line_items'])){
            // Handling duplicate line in properties
            $rearrangeLineItems = array();
            $i = 0;
            $j= 1;
            foreach ($reqObj['order']['line_items'] as $key => $itemValue){
                if (!isset($itemValue['utm'])){
                    if (isset($rearrangeLineItems[$itemValue['variant_id']])){
                        $rearrangeLineItems[$itemValue['variant_id']]['quantity'] += $itemValue['quantity']; 
                    } else {
                        $itemValue['name'] = 'Vitamin'.($j<10? '0'.$j++:$j++);
                        $rearrangeLineItems[$itemValue['variant_id']] = $itemValue;
                    }
                } else {
                    $rearrangeLineItems['utm'] = $itemValue;
                }
                $i++;
            }
            if(isset($reqObj['order']['Pack'])){
                $rearrangeLineItems['packs'] = array('packs' => $reqObj['order']['Pack']);
            }
            // print_r($rearrangeLineItems); die;
            unset($reqObj['order']['line_items']);
            $reqObj['order']['line_items'] = array_values($rearrangeLineItems);
            
            $totalPrice = 0;
            foreach ($reqObj['order']['line_items'] as $key => $itemValue){
                $propertyValue = '';
                if (!isset($itemValue['utm']) && !isset($itemValue['packs'])){

                    $productId = $itemValue['product_id'];
                    $variantId = $itemValue['variant_id'];
                    if (isset($prod['products'][$productId]) && isset($prod['products'][$productId]['variants'][$variantId])){
                        $variantData = $prod['products'][$productId]['variants'][$variantId];
                        $lineItemDosage = !empty($itemValue['dosage']) ? $itemValue['dosage'] : 'default title';
                        $propertyValue .= $itemValue['value']. '|' .$lineItemDosage. '|' .$itemValue['quantity']. '|' .$itemValue['variant_id']. '|' .$variantData['sku'];
                        $lineProperties[] = ['name' => $itemValue['name'],'value' => $propertyValue];
                        $lineItemQuantity = !empty($itemValue['quantity']) ? $itemValue['quantity'] : '1';
                        $totalPrice += $lineItemQuantity * $variantData['price'];
                    }
                } else if (isset($itemValue['utm'])){
                    $lineProperties[] = ['name'=> 'utm','value'=> $itemValue['utm']];
                } else if (isset($itemValue['packs'])){
                    $lineProperties[] = ['name'=> 'Packs','value'=> $itemValue['packs']];
                }
                
            }
        }
    $shippingAddress = $reqObj['order']['shipping_address'];
    $request['customer']['first_name'] = $shippingAddress['first_name'];
    $request['customer']['last_name'] = $shippingAddress['last_name'];
    $request['customer']['email'] = $reqObj['order']['email'];
    $request['email'] = $reqObj['order']['email'];
    $request['tags'] = 'influencer';
    $request['line_items'] = array(
        array(
            'title' => "Your Monthly Subscription (Ships every 7 Days)",
            'product_id' => $reqObj['order']['product_id'],
            'variant_id' => $reqObj['order']['variant_id'],
            'quantity'   => !empty($reqObj['order']['quantity']) ? $reqObj['order']['quantity'] : '1',
            'price'      => $totalPrice,
            'properties' => $lineProperties
        )
    );

    $request['shipping_address'] = array(
        "address1"  => $shippingAddress['address1'],
        "address2"  => $shippingAddress['address2'],
        "city"      => $shippingAddress['city'],
        "country"   => $shippingAddress['country'],
        "first_name"=> $shippingAddress['first_name'],
        "last_name"  => $shippingAddress['last_name'],
        "province"  => $shippingAddress['state'],
        "zip"       => $shippingAddress['zip'],
        "phone"     => $shippingAddress['phone'],
    );
    return $request;
    }

    // To set the shopify product details
    private function setProductDetails() {

        $productDetails = $this->productObj->getShopifyProducts();
        $prod  = [];
        if (!empty($productDetails)) {
            $prod['products'] = [];
            $subscription_product = array('1809168695367','4359186350162');
            foreach ($productDetails['products'] as $products) {
                if(in_array($products['id'],$subscription_product)){
                    continue;
                }
                $vari = [];
                foreach ($products['variants'] as $variants) { 
                    $vari[$variants['id']] = ['id' => $variants['id'] ,'product_id'=> $variants['product_id'],"title" => $variants['title'],'sku' => $variants['sku'],'price' => $variants['price']];
                }
                $prod['products'][$products['id']] = ['id'=>$products['id'], 'title'=>$products['title'],'variants'=>$vari];
            }
        }
        return $prod;
    }
}
?>