<?php

namespace App\Jobs;

require_once(dirname(__FILE__).'/../../vendor/autoload.php');

use Illuminate\Bus\Queueable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Support\Facades\File;
use Config;
use Log;

use OhMyBrew\BasicShopifyAPI;


class CreateShopifyCustomerJob implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    public $email;
    public $userName;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($email, $userName)
    {
        //
        $this->email = $email;
        $this->userName = $userName;
    }

    /**
     * Execute the job.
     * @return void
     */
    public function handle()
    {

        $api = new BasicShopifyAPI();

        $password   = Config::get('shopify.password');
        $shopDomain = Config::get('shopify.shopDomain');

        $api->setShop($shopDomain);
        $api->setAccessToken($password);
      
        $params = array(
            'customer' => array(
                'email' => $this->email,
                'first_name' => $this->userName 
            )
        );

        $try = 1;
        $retry = false;
        do{
            $result = $api->rest('POST', '/admin/customers.json', $params);
            if(!isset($result->body->customer->id)){
                $retry = true;
                $try++;
            }
        } while($retry && $try <=3);

        if(empty($result->errors))  //success
        {
            // TO DO
        }
        else  //error
        {
            // TO DO
        }

    }
}
