<?php

namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Config;
use OhMyBrew\BasicShopifyAPI;
use DB;
use Log;


class GenerateUserTokenJob implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;


    public $webhook;
    public $randString;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($webhook, $randString)
    {
        $this->webhook = $webhook;
        $this->randString = $randString;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        $webhookDetails = json_decode($this->webhook);
        
        $api = new BasicShopifyAPI();

        $password   = Config::get('shopify.password');
        $shopDomain = Config::get('shopify.shopDomain');

        $api->setShop($shopDomain);
        $api->setAccessToken($password);

        $email = $webhookDetails->email;
        $customerId  = $webhookDetails->id;

        $authToken = $this->randString;
        $metafield = array();

        $temp['key']        = 'authToken';
        $temp['value']      = $authToken;
        $temp['value_type'] = 'string';
        $temp['namespace']  = 'global';
        
        array_push($metafield, $temp);

        $user = DB::table('users')->where('user_email', '=', $email)->get();
        
        if(!isset($user[0]->user_email)){
            // create user in DB 
            $userId = DB::table('users')->insertGetId([
                'user_email' => $email,
                'status' => 1,
                'auth_token' => $authToken
            ]);
        }
        else
        {
            $userId = DB::table('users')->where('user_email', $email)->update(['auth_token' => $authToken]);  
            
        }
        // update user 
        $params = array(
            'customer' => array(
                'id' => $customerId,
                'metafields' => $metafield
            )
        );

        $method = 'PUT';
        $url = '/admin/customers/'.$customerId.'.json';

        $result = $api->rest($method, $url, $params); 

        $try = 1;
        $retry = false;
        do{
            $result = $api->rest($method, $url, $params); 
            if(!isset($result->body->customer->id)){
                $retry = true;
                $try++;
            }
        } while($retry && $try <=3);

        if(empty($result->errors))  //success
        {
            // TO DO
        }
        else  //error
        {
            // TO DO
        }


    }
}
