<?php 

namespace App\Model\Shopify;
use Illuminate\Database\Eloquent\Model;
use Config;

class Product extends Model{
    
 
    public $shop_domain;
    private $token;
    private $api_key;
    private $secret;
    protected $global;
    protected $apiCount;
    
    function __construct(){

        //$this->setModel();

        $this->api_key  = Config::get('shopify.apiKey');
		$this->token    =  Config::get('shopify.password');
		$this->secret   = Config::get('shopify.secret');
		$this->shop_domain   =  Config::get('shopify.shopDomain');

    }


    
    /**
     * Getting shopify products based on productID and variantID
     */
     function getShopifyProductByID($productID, $variantID){

        $method = 'GET';
        //$path   = '/admin/products.json?handle=20x20-avalon-lotus-embroidery';
        // $path   = '/admin/products.json?ids=4016443261016,4003323150424';
        $path   = '/admin/products/'.$productID.'/variants/'.$variantID.'.json';

        $products = $this->call($method, $path);
        if($products){
            return  $products;
        }
        return null;
    }

      
    /**
     * Getting shopify products based on page no. and limit in api call
     */
    function getShopifyProducts($pageNo = NULL , $limit = NULL){
        $method = 'GET';
        $path   = '/admin/products.json?published_status=published&fields=id,title,variants';

        $products = $this->call($method, $path);
        if($products){
            return  $products;
        }
        return null;
    }
    

     function call($method, $path, $params=array())
    {

        /****************api limit control**********************************/
        $duration = 1;
        $apiLimit = 2;
        if($this->global['apiRequestTime'] == 0)
        {        
            $this->global['apiRequestTime'] = strtotime("now");
        }
        else{
            $timeDifference = strtotime("now")- $this->global['apiRequestTime'];
            if ($timeDifference< $duration)  {
                if ( $this->apiCount >= $apiLimit ) {
                    $sleepTime = (int)ceil($duration-$timeDifference);
                    sleep($sleepTime);  // this step will wait for next api request.
                    $this->global['apiRequestTime'] = strtotime("now");
                    $this->apiCount = 0;       // next cycle starts
                }
            }else {
                $this->global['apiRequestTime'] = strtotime("now");
                $this->apiCount = 0; 
            }
        }        
        $this->apiCount++;
        //echo PHP_EOL."count".$count;
        
           /**************************************************/
        $baseurl = "https://{$this->shop_domain}/";
    
        $url = $baseurl.ltrim($path, '/');
        $query = in_array($method, array('GET','DELETE')) ? $params : array();
        $payload = in_array($method, array('POST','PUT')) ? json_encode($params) : array();
        $request_headers = in_array($method, array('POST','PUT')) ? array("Content-Type: application/json; charset=utf-8", 'Expect:') : array();

        // add auth headers
        $request_headers[] = 'X-Shopify-Access-Token: ' . $this->token;

        $response = $this->curlHttpApiRequest($method, $url, $query, $payload, $request_headers);
        $response = json_decode($response, true);
        
        return $response;
    }

     function curlHttpApiRequest($method, $url, $query='', $payload='', $request_headers=array())
    {

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_ENCODING, '');
        curl_setopt($curl, CURLOPT_MAXREDIRS, 10);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);

        if ($method !== 'GET') {

            curl_setopt($curl, CURLOPT_POSTFIELDS, $payload);
        }

        curl_setopt($curl, CURLOPT_HTTPHEADER, $request_headers);
     
        $response = curl_exec($curl);
        return $response;

    }

}

//new shopifyProduct();
